@@ -21,17 +21,16 @@
 
 namespace ranger {
 
-TreeSurvival::TreeSurvival(std::vector<double>* unique_timepoints, size_t status_varID,
-    std::vector<size_t>* response_timepointIDs) :
-    status_varID(status_varID), unique_timepoints(unique_timepoints), response_timepointIDs(response_timepointIDs), num_deaths(
-        0), num_samples_at_risk(0) {
+TreeSurvival::TreeSurvival(std::vector<double>* unique_timepoints, std::vector<size_t>* response_timepointIDs) :
+    unique_timepoints(unique_timepoints), response_timepointIDs(response_timepointIDs), num_deaths(0), num_samples_at_risk(
+        0) {
   this->num_timepoints = unique_timepoints->size();
 }
 
 TreeSurvival::TreeSurvival(std::vector<std::vector<size_t>>& child_nodeIDs, std::vector<size_t>& split_varIDs,
     std::vector<double>& split_values, std::vector<std::vector<double>> chf, std::vector<double>* unique_timepoints,
     std::vector<size_t>* response_timepointIDs) :
-    Tree(child_nodeIDs, split_varIDs, split_values), status_varID(0), unique_timepoints(unique_timepoints), response_timepointIDs(
+    Tree(child_nodeIDs, split_varIDs, split_values), unique_timepoints(unique_timepoints), response_timepointIDs(
         response_timepointIDs), chf(chf), num_deaths(0), num_samples_at_risk(0) {
   this->num_timepoints = unique_timepoints->size();
 }
@@ -74,7 +73,7 @@ void TreeSurvival::computeSurvival(size_t nodeID) {
   chf[nodeID] = chf_temp;
 }
 
-double TreeSurvival::computePredictionAccuracyInternal() {
+double TreeSurvival::computePredictionAccuracyInternal(std::vector<double>* prediction_error_casewise) {
 
   // Compute summed chf for samples
   std::vector<double> sum_chf;
@@ -84,11 +83,32 @@ double TreeSurvival::computePredictionAccuracyInternal() {
   }
 
   // Return concordance index
-  return computeConcordanceIndex(*data, sum_chf, dependent_varID, status_varID, oob_sampleIDs);
+  return computeConcordanceIndex(*data, sum_chf, oob_sampleIDs, prediction_error_casewise);
 }
 
 bool TreeSurvival::splitNodeInternal(size_t nodeID, std::vector<size_t>& possible_split_varIDs) {
 
+  // Stop if node is pure
+  bool pure = true;
+  double pure_time = 0;
+  double pure_status = 0;
+  for (size_t pos = start_pos[nodeID]; pos < end_pos[nodeID]; ++pos) {
+    size_t sampleID = sampleIDs[pos];
+    double time = data->get_y(sampleID, 0);
+    double status = data->get_y(sampleID, 1);
+    if (pos != start_pos[nodeID] && (time != pure_time || status != pure_status)) {
+      pure = false;
+      break;
+    }
+    pure_time = time;
+    pure_status = status;
+  }
+  if (pure) {
+    computeDeathCounts(nodeID);
+    computeSurvival(nodeID);
+    return true;
+  }
+
   if (splitrule == MAXSTAT) {
     return findBestSplitMaxstat(nodeID, possible_split_varIDs);
   } else if (splitrule == EXTRATREES) {
@@ -147,6 +167,9 @@ bool TreeSurvival::findBestSplit(size_t nodeID, std::vector<size_t>& possible_sp
       addImpurityImportance(nodeID, best_varID, best_decrease);
     }
 
+    // Regularization
+    saveSplitVarID(best_varID);
+
     return false;
   }
 }
@@ -169,11 +192,10 @@ bool TreeSurvival::findBestSplitMaxstat(size_t nodeID, std::vector<size_t>& poss
   status.reserve(num_samples_node);
   for (size_t pos = start_pos[nodeID]; pos < end_pos[nodeID]; ++pos) {
     size_t sampleID = sampleIDs[pos];
-    time.push_back(data->get(sampleID, dependent_varID));
-    status.push_back(data->get(sampleID, status_varID));
+    time.push_back(data->get_y(sampleID, 0));
+    status.push_back(data->get_y(sampleID, 1));
   }
   std::vector<double> scores = logrankScores(time, status);
-  //std::vector<double> scores = logrankScoresData(data, dependent_varID, status_varID, sampleIDs[nodeID]);
 
   // Save split stats
   std::vector<double> pvalues;
@@ -193,7 +215,7 @@ bool TreeSurvival::findBestSplitMaxstat(size_t nodeID, std::vector<size_t>& poss
     x.reserve(num_samples_node);
     for (size_t pos = start_pos[nodeID]; pos < end_pos[nodeID]; ++pos) {
       size_t sampleID = sampleIDs[pos];
-      x.push_back(data->get(sampleID, varID));
+      x.push_back(data->get_x(sampleID, varID));
     }
 
     // Order by x
@@ -286,7 +308,7 @@ void TreeSurvival::computeDeathCounts(size_t nodeID) {
 
   for (size_t pos = start_pos[nodeID]; pos < end_pos[nodeID]; ++pos) {
     size_t sampleID = sampleIDs[pos];
-    double survival_time = data->get(sampleID, dependent_varID);
+    double survival_time = data->get_y(sampleID, 0);
 
     size_t t = 0;
     while (t < num_timepoints && (*unique_timepoints)[t] < survival_time) {
@@ -297,7 +319,7 @@ void TreeSurvival::computeDeathCounts(size_t nodeID) {
     // Now t is the survival time, add to at risk and to death if death
     if (t < num_timepoints) {
       ++num_samples_at_risk[t];
-      if (data->get(sampleID, status_varID) == 1) {
+      if (data->get_y(sampleID, 1) == 1) {
         ++num_deaths[t];
       }
     }
@@ -311,7 +333,7 @@ void TreeSurvival::computeChildDeathCounts(size_t nodeID, size_t varID, std::vec
   // Count deaths in right child per timepoint and possbile split
   for (size_t pos = start_pos[nodeID]; pos < end_pos[nodeID]; ++pos) {
     size_t sampleID = sampleIDs[pos];
-    double value = data->get(sampleID, varID);
+    double value = data->get_x(sampleID, varID);
     size_t survival_timeID = (*response_timepointIDs)[sampleID];
 
     // Count deaths until split_value reached
@@ -320,7 +342,7 @@ void TreeSurvival::computeChildDeathCounts(size_t nodeID, size_t varID, std::vec
       if (value > possible_split_values[i]) {
         ++num_samples_right_child[i];
         ++delta_samples_at_risk_right_child[i * num_timepoints + survival_timeID];
-        if (data->get(sampleID, status_varID) == 1) {
+        if (data->get_y(sampleID, 1) == 1) {
           ++num_deaths_right_child[i * num_timepoints + survival_timeID];
         }
       } else {
@@ -392,6 +414,9 @@ void TreeSurvival::findBestSplitValueLogRank(size_t nodeID, size_t varID, double
       logrank = fabs(numerator / sqrt(denominator_squared));
     }
 
+    // Regularization
+    regularize(logrank, varID);
+
     if (logrank > best_logrank) {
       best_value = (possible_split_values[i] + possible_split_values[i + 1]) / 2;
       best_varID = varID;
@@ -420,7 +445,7 @@ void TreeSurvival::findBestSplitValueLogRankUnordered(size_t nodeID, size_t varI
   }
 
   // Number of possible splits is 2^num_levels
-  size_t num_splits = (1 << factor_levels.size());
+  size_t num_splits = (1ULL << factor_levels.size());
 
   // Compute logrank test statistic for each possible split
   // Split where all left (0) or all right (1) are excluded
@@ -430,10 +455,10 @@ void TreeSurvival::findBestSplitValueLogRankUnordered(size_t nodeID, size_t varI
     // Compute overall splitID by shifting local factorIDs to global positions
     size_t splitID = 0;
     for (size_t j = 0; j < factor_levels.size(); ++j) {
-      if ((local_splitID & (1 << j))) {
+      if ((local_splitID & (1ULL << j))) {
         double level = factor_levels[j];
         size_t factorID = floor(level) - 1;
-        splitID = splitID | (1 << factorID);
+        splitID = splitID | (1ULL << factorID);
       }
     }
 
@@ -448,15 +473,15 @@ void TreeSurvival::findBestSplitValueLogRankUnordered(size_t nodeID, size_t varI
     for (size_t pos = start_pos[nodeID]; pos < end_pos[nodeID]; ++pos) {
       size_t sampleID = sampleIDs[pos];
       size_t survival_timeID = (*response_timepointIDs)[sampleID];
-      double value = data->get(sampleID, varID);
+      double value = data->get_x(sampleID, varID);
       size_t factorID = floor(value) - 1;
 
       // If in right child, count
       // In right child, if bitwise splitID at position factorID is 1
-      if ((splitID & (1 << factorID))) {
+      if ((splitID & (1ULL << factorID))) {
         ++num_samples_right_child;
         ++delta_samples_at_risk_right_child[survival_timeID];
-        if (data->get(sampleID, status_varID) == 1) {
+        if (data->get_y(sampleID, 1) == 1) {
           ++num_deaths_right_child[survival_timeID];
         }
       }
@@ -494,6 +519,9 @@ void TreeSurvival::findBestSplitValueLogRankUnordered(size_t nodeID, size_t varI
       logrank = fabs(numerator / sqrt(denominator_squared));
     }
 
+    // Regularization
+    regularize(logrank, varID);
+
     if (logrank > best_logrank) {
       best_value = splitID;
       best_varID = varID;
@@ -526,9 +554,9 @@ void TreeSurvival::findBestSplitValueAUC(size_t nodeID, size_t varID, double& be
   // For all pairs
   for (size_t k = start_pos[nodeID]; k < end_pos[nodeID]; ++k) {
     size_t sample_k = sampleIDs[k];
-    double time_k = data->get(sample_k, dependent_varID);
-    double status_k = data->get(sample_k, status_varID);
-    double value_k = data->get(sample_k, varID);
+    double time_k = data->get_y(sample_k, 0);
+    double status_k = data->get_y(sample_k, 1);
+    double value_k = data->get_x(sample_k, varID);
 
     // Count samples in left node
     for (size_t i = 0; i < num_splits; ++i) {
@@ -540,9 +568,9 @@ void TreeSurvival::findBestSplitValueAUC(size_t nodeID, size_t varID, double& be
 
     for (size_t l = k + 1; l < end_pos[nodeID]; ++l) {
       size_t sample_l = sampleIDs[l];
-      double time_l = data->get(sample_l, dependent_varID);
-      double status_l = data->get(sample_l, status_varID);
-      double value_l = data->get(sample_l, varID);
+      double time_l = data->get_y(sample_l, 0);
+      double status_l = data->get_y(sample_l, 1);
+      double value_l = data->get_x(sample_l, varID);
 
       // Compute split
       computeAucSplit(time_k, time_l, status_k, status_l, value_k, value_l, num_splits, possible_split_values,
@@ -557,6 +585,10 @@ void TreeSurvival::findBestSplitValueAUC(size_t nodeID, size_t varID, double& be
       continue;
     } else {
       double auc = fabs((num_count[i] / 2) / num_total[i] - 0.5);
+
+      // Regularization
+      regularize(auc, varID);
+
       if (auc > best_auc) {
         best_value = (possible_split_values[i] + possible_split_values[i + 1]) / 2;
         best_varID = varID;
@@ -682,6 +714,9 @@ bool TreeSurvival::findBestSplitExtraTrees(size_t nodeID, std::vector<size_t>& p
       addImpurityImportance(nodeID, best_varID, best_decrease);
     }
 
+    // Regularization
+    saveSplitVarID(best_varID);
+
     return false;
   }
 }
@@ -708,6 +743,9 @@ void TreeSurvival::findBestSplitValueExtraTrees(size_t nodeID, size_t varID, dou
   for (size_t i = 0; i < num_random_splits; ++i) {
     possible_split_values.push_back(udist(random_number_generator));
   }
+  if (num_random_splits > 1) {
+    std::sort(possible_split_values.begin(), possible_split_values.end());
+  }
 
   size_t num_splits = possible_split_values.size();
 
@@ -756,6 +794,9 @@ void TreeSurvival::findBestSplitValueExtraTrees(size_t nodeID, size_t varID, dou
       logrank = fabs(numerator / sqrt(denominator_squared));
     }
 
+    // Regularization
+    regularize(logrank, varID);
+
     if (logrank > best_logrank) {
       best_value = possible_split_values[i];
       best_varID = varID;
@@ -798,21 +839,21 @@ void TreeSurvival::findBestSplitValueExtraTreesUnordered(size_t nodeID, size_t v
 
     // Draw random subsets, sample all partitions with equal probability
     if (indices_in_node.size() > 1) {
-      size_t num_partitions = (2 << (indices_in_node.size() - 1)) - 2; // 2^n-2 (don't allow full or empty)
+      size_t num_partitions = (2ULL << (indices_in_node.size() - 1ULL)) - 2ULL; // 2^n-2 (don't allow full or empty)
       std::uniform_int_distribution<size_t> udist(1, num_partitions);
       size_t splitID_in_node = udist(random_number_generator);
       for (size_t j = 0; j < indices_in_node.size(); ++j) {
-        if ((splitID_in_node & (1 << j)) > 0) {
+        if ((splitID_in_node & (1ULL << j)) > 0) {
           split_subset.push_back(indices_in_node[j]);
         }
       }
     }
     if (indices_out_node.size() > 1) {
-      size_t num_partitions = (2 << (indices_out_node.size() - 1)) - 1; // 2^n-1 (allow full or empty)
+      size_t num_partitions = (2ULL << (indices_out_node.size() - 1ULL)) - 1ULL; // 2^n-1 (allow full or empty)
       std::uniform_int_distribution<size_t> udist(0, num_partitions);
       size_t splitID_out_node = udist(random_number_generator);
       for (size_t j = 0; j < indices_out_node.size(); ++j) {
-        if ((splitID_out_node & (1 << j)) > 0) {
+        if ((splitID_out_node & (1ULL << j)) > 0) {
           split_subset.push_back(indices_out_node[j]);
         }
       }
@@ -821,7 +862,7 @@ void TreeSurvival::findBestSplitValueExtraTreesUnordered(size_t nodeID, size_t v
     // Assign union of the two subsets to right child
     size_t splitID = 0;
     for (auto& idx : split_subset) {
-      splitID |= 1 << idx;
+      splitID |= 1ULL << idx;
     }
 
     // Initialize
@@ -835,15 +876,15 @@ void TreeSurvival::findBestSplitValueExtraTreesUnordered(size_t nodeID, size_t v
     for (size_t pos = start_pos[nodeID]; pos < end_pos[nodeID]; ++pos) {
       size_t sampleID = sampleIDs[pos];
       size_t survival_timeID = (*response_timepointIDs)[sampleID];
-      double value = data->get(sampleID, varID);
+      double value = data->get_x(sampleID, varID);
       size_t factorID = floor(value) - 1;
 
       // If in right child, count
       // In right child, if bitwise splitID at position factorID is 1
-      if ((splitID & (1 << factorID))) {
+      if ((splitID & (1ULL << factorID))) {
         ++num_samples_right_child;
         ++delta_samples_at_risk_right_child[survival_timeID];
-        if (data->get(sampleID, status_varID) == 1) {
+        if (data->get_y(sampleID, 1) == 1) {
           ++num_deaths_right_child[survival_timeID];
         }
       }
@@ -881,6 +922,9 @@ void TreeSurvival::findBestSplitValueExtraTreesUnordered(size_t nodeID, size_t v
       logrank = fabs(numerator / sqrt(denominator_squared));
     }
 
+    // Regularization
+    regularize(logrank, varID);
+
     if (logrank > best_logrank) {
       best_value = splitID;
       best_varID = varID;
@@ -893,11 +937,6 @@ void TreeSurvival::addImpurityImportance(size_t nodeID, size_t varID, double dec
 
   // No variable importance for no split variables
   size_t tempvarID = data->getUnpermutedVarID(varID);
-  for (auto& skip : data->getNoSplitVariables()) {
-    if (tempvarID >= skip) {
-      --tempvarID;
-    }
-  }
 
   // Subtract if corrected importance and permuted variable, else add
   if (importance_mode == IMP_GINI_CORRECTED && varID >= data->getNumCols()) {
