@@ -1,54 +1,44 @@
 /*-------------------------------------------------------------------------------
- This file is part of Ranger.
+ This file is part of ranger.
 
- Ranger is free software: you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation, either version 3 of the License, or
- (at your option) any later version.
+ Copyright (c) [2014-2018] [Marvin N. Wright]
 
- Ranger is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- GNU General Public License for more details.
+ This software may be modified and distributed under the terms of the MIT license.
 
- You should have received a copy of the GNU General Public License
- along with Ranger. If not, see <http://www.gnu.org/licenses/>.
-
- Written by:
-
- Marvin N. Wright
- Institut für Medizinische Biometrie und Statistik
- Universität zu Lübeck
- Ratzeburger Allee 160
- 23562 Lübeck
- Germany
-
- http://www.imbs-luebeck.de
+ Please note that the C++ core of ranger is distributed under MIT license and the
+ R package "ranger" under GPL3 license.
  #-------------------------------------------------------------------------------*/
 
 #ifndef TREESURVIVAL_H_
 #define TREESURVIVAL_H_
 
+#include <vector>
+
 #include "globals.h"
 #include "Tree.h"
 
+namespace ranger {
+
 class TreeSurvival: public Tree {
 public:
-  TreeSurvival(std::vector<double>* unique_timepoints, size_t status_varID, std::vector<size_t>* response_timepointIDs);
+  TreeSurvival(std::vector<double>* unique_timepoints, std::vector<size_t>* response_timepointIDs);
 
   // Create from loaded forest
   TreeSurvival(std::vector<std::vector<size_t>>& child_nodeIDs, std::vector<size_t>& split_varIDs,
       std::vector<double>& split_values, std::vector<std::vector<double>> chf, std::vector<double>* unique_timepoints,
       std::vector<size_t>* response_timepointIDs);
 
-  virtual ~TreeSurvival();
+  TreeSurvival(const TreeSurvival&) = delete;
+  TreeSurvival& operator=(const TreeSurvival&) = delete;
 
-  void allocateMemory();
+  virtual ~TreeSurvival() override = default;
 
-  void appendToFileInternal(std::ofstream& file);
+  void allocateMemory() override;
+
+  void appendToFileInternal(std::ofstream& file) override;
   void computePermutationImportanceInternal(std::vector<std::vector<size_t>>* permutations);
 
-  const std::vector<std::vector<double> >& getChf() const {
+  const std::vector<std::vector<double>>& getChf() const {
     return chf;
   }
 
@@ -63,11 +53,11 @@ public:
 
 private:
 
-  void createEmptyNodeInternal();
+  void createEmptyNodeInternal() override;
   void computeSurvival(size_t nodeID);
-  double computePredictionAccuracyInternal();
-
-  bool splitNodeInternal(size_t nodeID, std::vector<size_t>& possible_split_varIDs);
+  double computePredictionAccuracyInternal(std::vector<double>* prediction_error_casewise) override;
+  
+  bool splitNodeInternal(size_t nodeID, std::vector<size_t>& possible_split_varIDs) override;
 
   bool findBestSplit(size_t nodeID, std::vector<size_t>& possible_split_varIDs);
   bool findBestSplitMaxstat(size_t nodeID, std::vector<size_t>& possible_split_varIDs);
@@ -80,11 +70,12 @@ private:
 
   void computeDeathCounts(size_t nodeID);
   void computeChildDeathCounts(size_t nodeID, size_t varID, std::vector<double>& possible_split_values,
-      size_t* num_samples_right_child, size_t* num_samples_at_risk_right_child, size_t* num_deaths_right_child,
-      size_t num_splits);
+      std::vector<size_t>& num_samples_right_child, std::vector<size_t>& num_samples_at_risk_right_child,
+      std::vector<size_t>& num_deaths_right_child, size_t num_splits);
 
   void computeAucSplit(double time_k, double time_l, double status_k, double status_l, double value_k, double value_l,
-      size_t num_splits, std::vector<double>& possible_split_values, double* num_count, double* num_total);
+      size_t num_splits, std::vector<double>& possible_split_values, std::vector<double>& num_count,
+      std::vector<double>& num_total);
 
   void findBestSplitValueLogRank(size_t nodeID, size_t varID, double& best_value, size_t& best_varID,
       double& best_logrank);
@@ -99,26 +90,26 @@ private:
 
   void addImpurityImportance(size_t nodeID, size_t varID, double decrease);
 
-  void cleanUpInternal() {
-    delete[] num_deaths;
-    delete[] num_samples_at_risk;
+  void cleanUpInternal() override {
+    num_deaths.clear();
+    num_deaths.shrink_to_fit();
+    num_samples_at_risk.clear();
+    num_samples_at_risk.shrink_to_fit();
   }
 
-  size_t status_varID;
-
   // Unique time points for all individuals (not only this bootstrap), sorted
-  std::vector<double>* unique_timepoints;
+  const std::vector<double>* unique_timepoints;
   size_t num_timepoints;
-  std::vector<size_t>* response_timepointIDs;
+  const std::vector<size_t>* response_timepointIDs;
 
   // For all terminal nodes CHF for all unique timepoints. For other nodes empty vector.
   std::vector<std::vector<double>> chf;
 
   // Fields to save to while tree growing
-  size_t* num_deaths;
-  size_t* num_samples_at_risk;
-
-  DISALLOW_COPY_AND_ASSIGN(TreeSurvival);
+  std::vector<size_t> num_deaths;
+  std::vector<size_t> num_samples_at_risk;
 };
 
+} // namespace ranger
+
 #endif /* TREESURVIVAL_H_ */
